@@ -13,16 +13,8 @@ Requires numpy and ctypes. Tested under windows only.
 """
 
 import sys
-try:
-    import ctypes as ct
-except ImportError:
-    print("this module requires ctypes")
-    sys.exit(-1)
-try:
-    import numpy as np
-except ImportError:
-    print("this module requires numpy")
-    sys.exit(-1)
+import ctypes as ct
+import numpy as np
 
 if sys.platform == "win32" :
     dllName = 'libsndfile-1'
@@ -47,9 +39,7 @@ except:
         dllPath = os.path.dirname(os.path.abspath(__file__))
         _lib = ct.CDLL(os.path.join(dllPath, dllName))
     except:
-        import sys
-        print("could not import libsndfile dll, make sure the dll '%s' is in the path"%(dllName))
-        sys.exit(-1)
+        raise Exception("could not import libsndfile dll, make sure the dll '%s' is in the path"%(dllName))
 
 _lib.sf_version_string.restype = ct.c_char_p
 _lib.sf_version_string.argtypes = None
@@ -258,12 +248,14 @@ class OPEN_MODES():
     SFM_WRITE = 0x20
     SFM_RDWR  = 0x30
 
+# deprecated
 class SEEK_MODES():
     #stdio.h :
     SEEK_SET = 0
     SEEK_CUR = 1
     SEEK_END = 2
 
+# deprecated (Code kept as reference of some functionalities not yet implemented in python-wavefile)
 class SndFile(object):
     """ Main Class of the wrapper, provides easy access to audio file contents """
 
